COUNTRY_LEGAL_ENTITY_TYPES_MAPPING = {
    "Albania": ["sh.a.", "sh.p.k."],
    "Argentina": [
        "s.a.",
        "s.r.l.",
        "s.c.p.a",
        "scpa",
        "s.c.e i.",
        "s.e.",
        "s.g.r",
        "soc.col.",
        "s.a.s.",
        "s.a.u.",
        "scs",
        "s.c.e l.",
    ],
    "Australia": ["nl", "pty. ltd.", "pty ltd", "llp", "ilp", "inc.", "trust"],
    "Austria": [
        "e.u.",
        "stg",
        "gesbr",
        "a.g.",
        "ag",
        "og",
        "kg",
        "gen",
        "privatstiftung",
        "gmbh",
    ],
    "Belarus": ["aat", "3at", "sat", "pup", "taa"],
    "Belgium": [
        "esv",
        "vzw",
        "vof",
        "snc",
        "comm.v",
        "scs",
        "bvba",
        "sprl",
        "cvba",
        "cvoa",
        "sca",
        "sep",
        "gie",
        "asbl",
        "comm. va",
        "bv",
        "srl",
        "ebvba",
        "sprlu",
        "nv",
        "sa",
        "scrl",
        "scri",
    ],
    "Bosnia / Herzegovina": [
        "d.d.",
        "a.d.",
        "d.n.o.",
        "d.o.o.",
        "k.v.",
        "s.p.",
        "k.d.",
    ],
    "Brazil": ["ltda", "s.a.", "pllc", "ad", "adsitz", "ead", "et", "kd", "kda", "sd"],
    "Brunei": ["Sdn. Bhd.", "bhd."],
    "Bulgaria": ["ad", "adsits", "ead", "et", "kd", "kda", "sd", "eood", "ood"],
    "Cambodia": ["gp", "sm pte ltd.", "pte ltd.", "plc ltd.", "peec", "sp"],
    "Canada": ["gp", "lp", "sp"],
    "Chile": [
        "eirl",
        "s.a.",
        "sgr",
        "s.g.r.",
        "ltda",
        "s.p.a.",
        "sa",
        "s. en c.",
        "ltda.",
    ],
    "China": [
        "ltd.",
        "plc",
        "有限责任公司",
        "股份有限公司",
        "个人独资企业",
        "合伙制企业",
    ],
    "Columbia": ["s.a.", "e.u.", "s.a.s.", "suc. de descendants", "sca", "ltda"],
    "Costa Rica": ["s.a.", "s.r.l.", "ltda."],
    "Croatia": [
        "d.d.",
        "d.o.o.",
        "obrt",
        "j.d.o.o.",
        "j.t.d.",
        "k.d.",
        "giu",
        "zadruga",
        "obrt",
        "opg",
    ],
    "Cyprus": ["ltd", "plc", "coop", "sp", "sf", "publaw", "pf", "oth"],
    "Czech Republic": [
        "a.s.",
        "akc. spol.",
        "s.r.o.",
        "spol. s r.o.",
        "v.o.s.",
        "obch. spol.",
        "a spol.",
        "k.s.",
        "kom. spol.",
        "kom. spol.",
        "živnost",
        "s.p.",
        "příspěvková organizace",
        "z.s.",
    ],
    "Denmark": [
        "i/s",
        "a/s",
        "k/s",
        "p/s",
        "amba",
        "a.m.b.a.",
        "fmba",
        "f.m.b.a.",
        "smba",
        "s.m.b.a.",
        "g/s",
        "forening",
        "ivs",
        "aps",
    ],
    "Dominican Republic": [
        "c. por a.",
        "cxa",
        "s.a.",
        "s.a.s.",
        "srl.",
        "srl",
        "eirl.",
        "sa",
        "sas",
    ],
    "Ecuador": ["s.a.", "c.a.", "sa", "ep", "s.a.s.", "cia. ltda."],
    "Egypt": ["sae", "llc", "llp"],
    "Ethiopia": ["plc", "sc", "cs", "pe", "pc"],
    "Estonia": ["fie", "UÜ", "TÜ", "OÜ", "as", "MTÜ"],
    "Finland": [
        "t:mi",
        "tmi",
        "as oy",
        "as.oy",
        "ay",
        "ky",
        "oy",
        "oyj",
        "ok",
        "osk",
        "T:mi",
        "ry",
        "rp",
        "rs",
    ],
    "France": [
        "sicav",
        "sarl",
        "sogepa",
        "ei",
        "eurl",
        "sasu",
        "fcp",
        "gie",
        "sep",
        "snc",
        "scs",
        "sca",
        "scop",
        "sem",
        "sas",
        "ei",
        "fcp",
        "sppl",
        "sci",
        "sarl",
        "eurl",
        "sa",
        "scop",
        "sem",
        "sas",
        "sasu",
    ],
    "Germany": [
        "gmbh & co. kg",
        "e.g.",
        "e.v.",
        "gbr",
        "ohg",
        "partg",
        "kgaa",
        "gmbh",
        "g.m.b.h.",
        "ag",
        "mbh & co. kg",
        "kg",
        "AG & Co. KG",
        "SE & Co. KG",
        "GmbH & Co. OHG",
        "Limited & Co. KG",
        "PLC & Co. KG",
        "ApS & Co. KG",
        "LLC & Co. KG",
        "PartGmbBH",
        "PartG",
        "e.V.",
        "gGmbH",
        "UG",
        "e.G.",
    ],
    "Ghana": ["llc", "clg", "ulc"],
    "Greece": [
        "a.e.",
        "ae",
        "e.e.",
        "ee",
        "epe",
        "e.p.e.",
        "mepe",
        "m.e.p.e.",
        "o.e.",
        "oe",
        "ovee",
        "o.v.e.e.",
        "A.V.E.E.",
        "I.K.E.",
    ],
    "Guatemala": [
        "s.a.",
        "sa",
        "y Cía. S. C.",
        "y Cía. S. en C.",
        "Cía. Ltda.",
        "Cía. S. C. A",
    ],
    "Haiti": ["sa"],
    "Hong Kong": ["ltd", "unltd", "ultd", "limited"],
    "Hungary": [
        "e.v.",
        "e.c.",
        "bt.",
        "kft.",
        "kht.",
        "kkt.",
        "k.v.",
        "zrt.",
        "nyrt",
        "ev",
        "ec",
        "rt.",
        "szöv.",
    ],
    "Iceland": [
        "ehf.",
        "hf.",
        "ohf.",
        "s.f.",
        "ses.",
        "ehf. einkahlutafélag",
        "einstaklingsfyrirtæki",
        "hf. hlutafélag",
        "ohf. opinbert hlutafélag",
        "slf. samlagsfélag",
        "svf. samvinnufélag",
        "sf. sameignarfélag",
        "ses. sjálfseignarstofnun",
    ],
    "India": ["pvt. ltd.", "ltd.", "psu", "pse", "gcmmf"],
    "Indonesia": ["ud", "fa", "pt", "vof/fa", "mts", "cv", "pt tbk"],
    "Iran": [
        "شرکت سهامی عام",
        "شرکت سهامی خاص",
        "شرکت با مسئولیت محدود",
        "شرکت مختلط غیر سهامی",
        "شرکت مختلط سهامی",
        "شرکت مختلط سهامی",
        "شرکت تضامنی",
        "شرکت نسبی",
        "شرکت تعاونی تولید و مصرف",
    ],
    "Ireland": ["cpt", "teo", "ucits", "dac", "spv"],
    "Israel": ["b.m.", "bm", "ltd", "limited", "cc"],
    "Italy": [
        "s.n.c.",
        "s.a.s.",
        "s.p.a.",
        "s.a.p.a.",
        "s.r.l.",
        "s.c.r.l.",
        "s.s.",
        "s.c.p.a.",
        "s.c.a.r.l.",
        "s.r.l.s.",
    ],
    "Japan": [
        "k.k.",
        "co., ltd.",
        "company, limited",
        "g.k.",
        "y.k.",
        "gmk",
        "gsk",
        "nk",
        "tk",
        "株式会社",
        "合同会社",
        "有限会社",
        "合名会社",
        "合資会社",
        "任意組合",
        "匿名組合",
        "投資事業有限責任組合",
        "有限責任事業組合",
    ],
    "Kazakhstan": [
        "ao",
        "too",
        "tdo",
        "gp",
        "kt",
        "oo",
        "pt",
        "ptk",
        "prk",
        "po",
        "uchr",
    ],
    "Kosovo": ["b.i.", "o.p.", "sh.k.m.", "sh.p.k.", "j.s.c.", "sh.a."],
    "Latvia": ["as", "sia", "ik", "ps", "ks", "ĀKF", "bo", "vsia"],
    "Liechtenstein": ["ag", "gmbh"],
    "Lebanon": ["sal", "s.a.l.", "s.a.r.l.", "s.n.c.", "s.c.s.", "rr"],
    "Lithuania": ["uab", "ab", "ij", "mb", "VšĮ", "IĮ", "TŪB", "KŪB", "BĮ", "pp"],
    "Luxembourg": ["s.a.", "s.a.r.l.", "secs", "SARL-S", "scs", "scsp", "sca"],
    "Macedonia": ["d.o.o.", "d.o.o.e.l", "k.d.a.", "j.t.d.", "a.d.", "k.d."],
    "Malta": ["ltd", "plc"],
    "Malaysia": ["bhd.", "sdn. bhd.", "plt"],
    "Mexico": [
        "s.a.",
        "s. de. r.l.",
        "s. en c.",
        "s.a.b.",
        "s.a.p.i.",
        "S. en C. por A.",
    ],
    "Monaco": ["sam", "sarl", "snc", "scs", "sca"],
    "Mongolia": ["xk", "xxk"],
    "Montenegro": ["o.d.", "k.d.", "a.d.", "d.o.o."],
    "Morocco": ["s.a.", "s.a.s.", "s.a.r.l.", "g.i.e.", "s.n.c.", "s.c.s.", "s.c.a."],
    "Netherlands": [
        "v.o.f.",
        "c.v.",
        "b.v.",
        "n.v.",
        "w.a.",
        "b.a.",
        "u.a.",
        "vve",
        "mts",
        "vof",
        "cv",
        "bv",
        "nv",
    ],
    "New Zealand": ["tapui", "ltd", "limited"],
    "Nicaragua": ["s.a.", "cia", "cia. ltda.", "s.c.s.", "s.c.a."],
    "Nigeria": ["gte.", "plc", "ltd.", "ultd.", "ltd./gte.", "llp"],
    "Norway": [
        "asa",
        "as",
        "ans",
        "ba",
        "bl",
        "da",
        "etat",
        "fkf",
        "hf",
        "iks",
        "kf",
        "ks",
        "nuf",
        "rhf",
        "sf",
        "ub",
    ],
    "North Macedonia": ["a.d.", "d.o.o.", "d.o.o.e.l.", "k.d.", "k.d.a.", "j.t.d."],
    "Oman": ["saog", "saoc"],
    "Pakistan": ["ltd.", "pvt. ltd.", "ltd", "limited"],
    "Peru": ["sa", "s.a.", "s.a.a.", "s.a.c."],
    "Philippines": [
        "coop.",
        "co.",
        "opc",
        "corp.",
        "corp",
        "ent.",
        "inc.",
        "inc",
        "llc",
        "l.l.c.",
        "ltd.",
        "ltd. co.",
        "llp",
        "cia",
        "sa",
        "s.a.",
    ],
    "Poland": [
        "p.p.",
        "s.k.a.",
        "sp.j.",
        "sp.k.",
        "sp.p.",
        "sp. z o.o.",
        "s.c.",
        "s.a.",
        "p.s.a.",
    ],
    "Portugal": ["lda.", "crl", "s.a.", "s.f.", "sgps", "sua", "sgoic", "sic"],
    "Romania": ["s.c.a.", "s.c.s.", "s.n.c.", "s.r.l.", "o.n.g.", "s.a.", "pfa"],
    "Russia": [
        "ooo",
        "oao",
        "zao",
        "3ao",
        "пао",
        "оао",
        "ооо",
        "ngo",
        "npo",
        "ip",
        "gp",
        "gup",
        "mp",
        "mup",
        "ao",
    ],
    "Saudi Arabia": [
        "شركة ذات مسئولية محدودة",
        "شركة مساهمة",
        "شركة تضامن",
        "شركة التوصية البسيطة",
        "شركة أجنبية ",
        "مؤسسة فردية",
    ],
    "Serbia": ["d.o.o.", "a.d.", "k.d.", "o.d.", "d.d.", "j.p."],
    "Singapore": [
        "bhd",
        "pte ltd",
        "sdn bhd",
        "llp",
        "l.l.p.",
        "ltd.",
        "pte",
        "pte. ltd.",
        "llp",
    ],
    "Slovenia": ["d.d.", "d.o.o.", "d.n.o.", "k.d.", "s.p."],
    "Slovakia": [
        "a.s.",
        "akc. spol.",
        "s.r.o.",
        "spol. s r.o.",
        "k.s.",
        "kom. spol.",
        "v.o.s.",
        "a spol.",
        "družstvo",
        "živnosť",
    ],
    "South Korea": [
        "합명회사",
        "합자회사",
        "유한책임회사",
        "주식회사;",
        "유한회사",
        "협동조합",
        "협동조합연합회",
    ],
    "South Africa": ["(pty) ltd", "ltd.", "cc"],
    "Spain": [
        "s.a.",
        "s.a.d.",
        "s.l.",
        "s.l.l.",
        "s.l.n.e.",
        "s.c.",
        "s.cra",
        "s.coop",
        "sal",
        "sccl",
        "c.b.",
        "s.c.p.",
        "S.Com. p. A.",
        "S.Com p.A.P.",
        "ute",
        "scr",
        "fcr",
        "sicc",
        "ficc",
        "sgeic",
    ],
    "Sweden": ["ab", "hb", "kb", "ek"],
    "Switzerland": [
        "ab",
        "sa",
        "gmbh",
        "g.m.b.h.",
        "sarl",
        "sagl",
        "ag",
        "sicaf",
        "wig",
        "ri",
        "eg",
        "koig",
        "kg",
        "komag",
    ],
    "Tunisia": ["sa", "sarl", "suarl"],
    "Thailand": [
        "บริษัทมหาชนจำกัด",
        "บริษัทเอกชนจำกัด",
        "ห้างหุ้นส่วนจำกัด",
        "ห้างหุ้นส่วนสามัญนิติบุคคล",
    ],
    "Taiwan": ["無限公司", "有限公司", "兩合公司", "股份有限公司"],
    "Turkey": [
        "koop.",
        "Adi Şirket",
        "Koll. Şti.",
        "Kom. Şti",
        "A.Ş. / A.O.",
        "Ltd. Şti. / L.Ş. / L.O.",
        "HAAO",
        "koop.",
        "Şahıs Firması",
        "Şb.",
    ],
    "Uganda": ["llc", "plc"],
    "Ukraine": [
        "dat",
        "fop",
        "kt",
        "pt",
        "tdv",
        "tov",
        "pp",
        "vat",
        "zat",
        "at",
        "pat",
        "prat",
        "at",
        "ДАТ",
        "ФОП",
        "ПT",
        "ТДВ",
        "TOB",
        "ПП",
        "ПАТ",
        "рАТ",
    ],
    "United Kingdom": [
        "plc.",
        "plc",
        "cic",
        "cio",
        "l.l.p.",
        "llp",
        "l.p.",
        "lp",
        "ltd.",
        "ltd",
        "limited",
        "slp",
        "ultd.",
        "unltd.",
    ],
    "United Arab Emirates": ["fze", "fzc", "llc"],
    "United States": [
        "llc",
        "inc.",
        "corporation",
        "incorporated",
        "company",
        "limited",
        "corp.",
        "inc.",
        "inc",
        "llp",
        "l.l.p.",
        "pllc",
        "and company",
        "& company",
        "inc",
        "inc.",
        "corp.",
        "corp",
        "ltd.",
        "ltd",
        "& co.",
        "& co",
        "co.",
        "co",
        "lp",
        "smllc",
        "pac",
        "fsb",
        "na",
        "NT&SA",
        "ncua",
        "lp",
        "lllp",
        "lc",
        "llc",
        "pllc",
        "l3c",
        "pbc",
        "pc",
        "p.c.",
        "co-op",
        "coop",
        "cp",
        "dba",
    ],
    "Uruguay": ["s.a.", "s.a.s.", "s.r.l.", "s.c.i.", "s.c."],
    "Uzbekistan": ["mchj", "qmj", "aj", "oaj", "yoaj", "xk", "xt", "ok", "uk", "qk"],
    "Venezuela": ["s.a.", "s.r.l.", "s.c.s.", "soc. col."],
    "Vietnam": [
        "Cty TNHH",
        "Cty TNHH MTV",
        "Cty CP",
        "DNNN",
        "DNTN",
        "DTNN",
        "htx",
        "Chi nhánh",
        "Nhóm Cty",
    ],
    "Zambia": ["ulc", "llc"],
    "Zimbabwe": ["plc", "pbc"],
}
